<?php

/**
 * @file
 * User authentication classes and interfaces.
 */

/**
 * Main class to deal with collected plugins.
 */
class UserAuthAPI {

  protected $plugins = array();
  protected $plugin_info = array();
  protected $form_values = array();
  protected $validation_result = NULL;
  protected $plugin_info_collected = FALSE;
  protected $plugins_instantiated = FALSE;

  function __construct($form_values = array()) {
    $this->form_values = $form_values;
  }

  /**
   * Searches available classes for ones implementing the 
   * UserAuthValidatePlugin interface and stores them as a ReflectionClass.
   */
  function getAvailablePluginInfo() {
    // Return plugins array if they have already been collected.
    if ($this->plugin_info_collected) {
      return $this->plugins;
    }

    // Get our plugins.
    ctools_include('plugins');
    $plugins = ctools_get_plugins('user_auth_api', 'user_auth_api');

    foreach ($plugins as $name => $info) {
      // Invoke reflection class to do our plugin search.
      if (class_exists($info['plugin'])) {
        $reflectionClass = new ReflectionClass($info['plugin']);
        if ($reflectionClass->implementsInterface('UserAuthAPIPlugin')) {
          $this->plugin_info[$name]['reflection'] = $reflectionClass;
          $this->plugin_info[$name]['info'] = $info;
        }
        else {
          throw new UnexpectedValueException;
        }
      }
      else {
        throw new UnexpectedValueException;
      }
    }
    $this->plugin_info_collected = TRUE;

    return $this->plugin_info;
  }

  /**
   * Iterate though $this->plugin_info as instantiate a new instance
   * of each plugin class.
   */
  function instantiatePlugins() {
    // Get plugin info if it hasn't already been collected.
    if (!$this->plugin_info_collected) {
      $this->getAvailablePluginInfo();
    }

    // Only collect plugin instances if they haven't been already.
    if (!$this->plugins_instantiated) {
      foreach ($this->plugin_info as $name => $plugin_info) {
        $plugin = $plugin_info['reflection']->newInstance();
        $this->plugins[$name] = array(
          'plugin' => $plugin,
          'weight' => $plugin->defaultWeight(),
          'info' => $plugin_info['info'],
        );
      }
      $this->plugins_instantiated = TRUE;
    }
    return $this->plugins;
  }

  /**
   * Sort plugins by weight and invoke validate method on each.
   */
  function invokePlugins($exclude = array()) {
    if (!$this->plugins_instantiated) {
      $this->instantiatePlugins();
    }

    $plugins = $this->sortPluginsByWeight();

    foreach ($plugins as $plugin_name => $plugin_info) {
      if (in_array($plugin_name, $exclude)) {
        continue;
      }
      $plugin = $plugin_info['plugin'];
      // Call the validate method on each available class.
      $validation_result = $plugin->validate($this->form_values);
      if (($validation_result === TRUE) || ($validation_result === FALSE)) {
        $this->validation_result = $validation_result;
        return new UserAuthAPIValidateResponse($validation_result, $plugin);
      }
    }

    return new UserAuthAPIValidateResponse($this->validation_result);
  }

  /**
   * Sort plugins array by weight method on each plugin.
   *
   * @return
   *  Sorted plugins array.
   */
  function sortPluginsByWeight() {
    if (empty($this->plugins)) {
      $this->invokePlugins();
    }

    $plugins = $this->plugins;

    drupal_alter('user_auth_api_plugin_info', $plugins);
    uasort($plugins, 'user_auth_api_plugin_sort');

    return $plugins;
  }

} // UserAuthAPI.

/**
 * Response object to return containing validation info and plugin.
 */
class UserAuthAPIValidateResponse {

  protected $validation_result = NULL;
  protected $plugin;

  function __construct($validation_result, $plugin = NULL) {
    $this->validation_result = $validation_result;
    $this->plugin = $plugin;
  }

  function getValidationResult() {
    return $this->validation_result;
  }

  function getPlugin() {
    return $this->plugin;
  }
  
} // UserAuthValidateResponse.

/**
 * Interface for defining validation plugins.
 */
interface UserAuthAPIPlugin {

  /**
   * Define whether to plugin uses external validation or not.
   * If yes, user_external_login_register will be called to validate accounts.
   */
  function isExternal();

  /**
   * Main method to decide validation result.
   */
  function validate($form_values);

  /**
   * This method will be called if validation fails.
   */
  function validateError($form, $form_values);

  /**
   * This method will be called if validation succeeds.
   */
  function validateSuccess($account, $form_values);

  /**
   * The default weight of the plugin. This affects the order it is called
   * if there are multiple plugins.
   */
  function defaultWeight();

} // UserAuthValidatePlugin
